# Relative_RGB_color_mask_maker 8 & 16bit Version
# Ver. 0.15  (2022.5)
# add tanceparency factor
# add luminosity mask (0.12)
# add Monochrome film support (0.13)
# add imp2 mixing ratio parameter (0.14) for preparing next version
# add Mask Inverting Option for Exclude Selected Area (0.15)
# (c)2021-22 Ohnishi, Yasuo
from ij import IJ, ImagePlus
from ij import WindowManager as WM
from ij.gui import GenericDialog, ImagePanel
from ij.io import DirectoryChooser, OpenDialog
# from ij.gui import GenericDialog
from ij import plugin
from ij.plugin import ChannelSplitter, Duplicator
from ij.plugin import RGBStackMerge, RGBStackConverter
from ij.plugin import Duplicator
from ij.plugin.filter import ThresholdToSelection
from ij.process import ImageConverter, ImageProcessor
from ij.process import ByteProcessor
from loci.plugins import BF
from java.awt.event import AdjustmentListener, ItemListener
import os
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')

#---Convert Value in 8, 16, 32bit mode
def convValue(Value, Mode):
#     Converting Luminance Value each in 8, 16, 32bit mode
        if Mode == "16-bit":
                Value = Value * 256
        elif Mode == "32-bit":
                Value = Value * 256 * 256
        else:
                Value = Value
        return Value

#--------------------------------------------
def grayscale_mask_maker(imp1, imp2, imp3, YTh, Mode, blInvert, Show, colorCh, TrFac, Gaussian, GaussianSigma, imp2MixRatio):
# this is making relative color mask routine
    """
    parameters
        imp1: Target Channel (B), imp2: Comparative Channel (G)
        imp3: Comparative Channnel 2 (R)
        YTh: Target Color Ch Threshold (originally Yellow Threshold)
        blInvert: Color invert Yes or No (originally invert for blue mask) 
        TrFac: Transparency Factor
        Gaussian: Apply or not Gaussian Blur
        GaussianSigma: Gaussian Blur Sigma
        imp2MixRatio: imp2 Mixing Ratio (to imp3) 0.0 to 1.0
    """
# Comprative (G) ch.1 x 0.5
    imp2_05 = Duplicator().run(imp2)
    Vstr = str(imp2MixRatio)  # imp2 mixing ratio
    IJ.run(imp2_05, "Multiply...", "value=" + Vstr)
    if Show == "Yes":
        imp2_05.show()
# Comprative (R) ch.2 ch. x 0.5
    imp3_05 = Duplicator().run(imp3) # imp3 mixing ratio
    Vstr = str(1.0 - imp2MixRatio)
    IJ.run(imp3_05, "Multiply...", "value=" + Vstr)
    if Show == "Yes":
        imp3_05.show()
# Comparative (G) Ch. 1 x 0.5 + (R) Ch.2 x 0.5
    ic = plugin.ImageCalculator()
    imp32 = ic.run("Add created", imp3_05, imp2_05)
# Close Windows
    if Show == "Yes":
        imp3.hide()
        imp2.hide()
        imp1.hide()
        imp32.show()
        imp2_05.hide()
        imp3_05.hide()
    imp2_05.close()
    imp3_05.close()
# (Ch1(R) x 0.5 + Ch2(G) x 0.5) - Target Ch (B)
    ic = plugin.ImageCalculator()
# Make Target Color (Yellow) Image Files 
# in each Threshold, Make 3 kinds of Image Files: x1, x2, x4 
    V = str(YTh)
# Target (Yellow) Mask Threshold
    V2 = str(convValue(abs(YTh), Mode)) # actual Yellow mask Threshold
    if blInvert == False:
        # for making Yellow mask
           # Set the Threshold in R + G Image
        if YTh < 0: # restrict less region for yellow mask
            print (YTh, "Subtract", V)
            IJ.run(imp32, "Subtract...","value=" + V2)
            V= "-" + V
        else:  #including more region for yellow mask
            print (YTh, "Add", V)
            IJ.run(imp32, "Add...", "value=" + V2)
            V= "+" + V
        impGraymask = ic.run("Subtract created", imp32, imp1) # Make Yellow Mask
        if colorCh == "Yellow/Blue":
            V = "Y" + V
        elif colorCh == "Cyan/Red":
            V = "Cy" + V
        elif colorCh == "Magenta/Green":
            V = "Mg" + V
    else:
     # for making inverse(Blue) mask
         # Set the Threshold in comparative Ch1 + Ch2 Image
        if YTh >= 0:  #including more range for blue mask
            print (YTh, "Subtract", V)
            IJ.run(imp32, "Subtract...","value=" + V2)
            V= "+" + V
        else: # restrict less range for blue mask
            print (YTh, "Add", V)
            IJ.run(imp32, "Add...", "value=" + V2)
            V= "-" + V
        impGraymask = ic.run("Subtract created", imp1, imp32) # Make Blue Mask
        if colorCh == "Yellow/Blue":
            V = "B" + V
        elif colorCh == "Cyan/Red":
            V = "R" + V
        elif colorCh == "Magenta/Green":
            V = "G" + V
    if Show == "Yes":
        impGraymask.show()  # Show the subtracted image in window
    if TrFac < 100: # making grayscale mask
        IJ.run(impGraymask, "Multiply...", "value=" + str(4 * TrFac)) # making mask more transparent (x 4)
    else: # if TrFac = 100, making relative color mask binary (calling subroutine)
        Gmaskimp = make_binary_mask(impGraymask, 1, convValue(255, Mode), Mode, Show)
        if Show == "Yes":
            impGraymask.hide()  # Hide the subtracted image in window
        impGraymask = Gmaskimp
#   apply Gaussian blur
#    if Gaussian == True:
#        IJ.run(impGraymask, "Gaussian Blur...", "sigma=" + str(GaussianSigma))
#   closing working windows
    imp32.hide()
    imp32.close()
    return impGraymask, V
    """
    return values
        impGraymask: mask image
        V: description for mask parameters
    """
#-------------------------------------------
def luminosity_mask_maker(imp1, LTh, Mode, NegaInvert, Show, TrFac):
    """
    parameters
        imp1: Grayscale image
        LTh: Zone lower Threshold
        NegaInvert: Nega invert True or False
        TrFac: Transparency Factor
    """
    if Show == "Yes":
        imp1.show()
# Make Grayscale Mask Image Files 
    V1 = str(LTh) # for threshold expression in file name
# Target (Yellow) Mask Threshold
    V2 = str(convValue(abs(LTh), Mode)) # actual Yellow mask Threshold
    IJ.run(imp1, "Subtract...","value=" + V2)
    if NegaInvert == True: # for making inverted Negative mask
        IJ.run(imp1, "Multiply...", "value=" + str(4 / TrFac)) # making mask more transparent (x 4 for Nega)
        imp1.getProcessor().invert()
        V1 = V1 + "_Nega"
        V = "_Nega_x "+ str(TrFac)
    else: # for making positive mask
        IJ.run(imp1, "Multiply...", "value=" + str(4 * TrFac)) # making mask more transparent (x 4)
        V1 = V1 + "_Posi"
        V = "_Posi_x" + str(TrFac)
    impGraymask = imp1
    if Show == "Yes":
        impGraymask.show()  # Show the subtracted image in window
    return impGraymask, V, V1
    """
    return values
        impGraymask: mask image
        V: description for mask parameters
        V1: description for mask parameters
    """
#-------------------------------------------
def make_binary_mask(impZone, zoneLTh, zoneHTh, Mode, Show):
    """
    parameters
       impZone: Original Ch.image
       zoneLTh: Lower threshold, zoneHTh: Upper threshold
       Mode: bit depth, Show: Show windows

    """
# make impZone for Grayscale 8 bit!!
    ImageConverter(impZone).setDoScaling(True)
#           it is needs for 16bit image to down convert to 8bit
    ImageConverter(impZone).convertToGray8()
    ip = impZone.getProcessor()
    mask = ByteProcessor(ip.width, ip.height)
    mask.setValue(255)
    if Show == "Yes":
        impZone.show()
# make Zone mask
    impZone.getProcessor().setThreshold(zoneLTh, zoneHTh, ImageProcessor.NO_LUT_UPDATE)
    print "Zone L & H", zoneLTh, zoneHTh
    roi = ThresholdToSelection.run(impZone)
    mask.setRoi(roi)
    mask.fill(roi)
    try:
        maskimp = ImagePlus("Mask", mask) 
#        only 8bit Zone mask made from ROI: Zone mask: maskimp
    except: # exception process for ROI missing
        IJ.error("Selected ROI (Region of Interest) is missing.")
        if Show == "Yes":
            impZone.hide()
        impZone.close()
        return
    if Show == "Yes":
        maskimp.show() # show Zone mask

# deepen Zone Mask bit depth  (8bit to 16 or 32 bit)      
    if Mode == "16-bit":
        ImageConverter(maskimp).convertToGray16()
    elif Mode == "32-bit":
        ImageConverter(maskimp).convertToGray32()
    IJ.run(maskimp, "Multiply...", "value=" + str(convValue(1, Mode)))
#   after deepen bit depth, Save Zone Mask
    return maskimp
    """
    return values
        maskimp: mask image
    """
#----------------------------------------------
### RGB Manipulation 
def RGBmanipulation(imp0, folder,filename, baseCh, parm, colorCh, stackSize):
# this is mask making & saving main routine
    """
    parameters
        imp0: original image
        folder: folder name of original image file
        filename: file name of original image file
        baseCh: channel name for deciding mask upper & lower thresholds
        parm: parameters array from main routine
        color Ch: channel name for making color mask image
        stackSize: Color file or MonoChrome / 1 is Mono 3 is Color
    """
#  input Parameters
    if parm is not None: # if parm is not null, then get paraneters...
        Mode, YTh, blInvert, zoneLTh, zoneHTh, TrFac, Show, Gaussian, GaussianSigma, MaskInvert = parm 
        """
        variables
           Mode: Image Format 16-bit or 8-bit
           YTh: yellow(or blue) value threshold of mask
           zoneLTh: lower threshold of mask
           zoneHTh: upper threshold of mask
           blinvert: make blue mask or not
           Show: working wndows Showing Mode
           Gaussian: Apply or not Gaussian Blur
           GaussianSigma: Gaussian Blur Sigma
           MaskInvert: Mask Inversion for excluding selected area (True / False)
        """
        imp0.show()
    impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
    for i in impsCh: # it needs to convert Grayscale image
        if Mode == "8-bit":
            ImageConverter(i).convertToGray8()
        elif Mode == "16-bit":
            ImageConverter(i).convertToGray16()
        else:
            ImageConverter(i).convertToGray32()
    if stackSize!=1: #if imp0 is color then...
## R ch.
        impR =  impsCh[0]
## G ch.
        impG =  impsCh[1]
## B ch. & B x 0.3
        impB =  impsCh[2]
# Show Windows
#    imp0.close()
    if Show == "Yes":
        if stackSize!=1: #if imp0 is color then...
            impR.show()
            impG.show()
            impB.show()
        imp0.hide()
    if colorCh == "Yellow/Blue":
        imp1 = impB
        imp2 = impG
        imp3 = impR
    elif colorCh == "Cyan/Red":
        imp1 = impR
        imp2 = impG
        imp3 = impB
    elif colorCh == "Magenta/Green":
        imp1 = impG
        imp2 = impR
        imp3 = impB
    elif colorCh == "Luminosity":
        imp1 = Duplicator().run(imp0)
        IJ.run(imp1, "Conversions...", "weighted")
        if Mode == "16-bit":
            ImageConverter(imp1).convertToGray16()
        elif Mode == "8-bit":
            ImageConverter(imp1).convertToGray8()
        imp1.getProcessor().setLut(None)

#-----------------------------------
    if colorCh != "Luminosity": # not Luminosity
        imp2MixRatio = 0.5 # define imp2 Mixing Ratio = 0.5 (2022.5)
# Make relative Color Mask
        impYmask, V = grayscale_mask_maker(imp1, imp2, imp3, YTh, Mode, blInvert, Show, colorCh, TrFac, Gaussian, GaussianSigma, imp2MixRatio)
#    save relative color Mask
#     Mask inversion (add 0.15)
        if MaskInvert == False: # Mask No Inversion
            IJ.saveAsTiff(impYmask, folder + filename + "_Mask_Thr_".decode('utf-8') + V.decode('utf-8') + "_x4x" + str(TrFac) + ". tif".decode('utf-8')) # Save the Subtracted image x 4
        else: # Mask Inversion
            IJ.run(impYmask, "Invert", "")
            IJ.saveAsTiff(impYmask, folder + filename + "_Mask_Thr_".decode('utf-8') + V.decode('utf-8') + "_x4x" + str(TrFac) + "_Inv. tif".decode('utf-8')) # Save the Subtracted image x 4
    else:
# Make Luminosity Mask
        impYmask, V, V1 = luminosity_mask_maker(imp1, zoneLTh, Mode, blInvert, Show, TrFac)
#    save Luminosity Mask
#     Mask inversion (add 0.15)
        if MaskInvert == False: # Mask No Inversion
            IJ.saveAsTiff(impYmask, folder + filename + "_Mask_Thr_".decode('utf-8') + V1.decode('utf-8') + "_x4x" + str(TrFac) + ". tif".decode('utf-8')) # Save the Subtracted image x 4 x transparency factor
        else: # Mask Inversion
            IJ.run(impYmask, "Invert", "")
            IJ.saveAsTiff(impYmask, folder + filename + "_Mask_Thr_".decode('utf-8') + V1.decode('utf-8') + "_x4x" + str(TrFac) + "_Inv. tif".decode('utf-8')) # Save the Subtracted image x 4 x transparency factor
#
# select base channel for deciding mask valid lightness range
#     impZone: base channel image 
    if baseCh == "Luminosity":
        impZone = Duplicator().run(imp0)
        IJ.run(impZone, "Conversions...", "weighted")
        chName ="Gray"
        ImageConverter(impZone).convertToRGB()
    elif baseCh == "Blue":
        impZone = Duplicator().run(impB)
        chName ="B"
    elif baseCh == "Red":
        impZone = Duplicator().run(impR)
        chName ="R"
    elif baseCh == "Green":
        impZone = Duplicator().run(impG)
        chName ="G"
    if stackSize !=1:
        impR.close()
        impG.close()
        impB.close()
    imp0.close()
#-----------------------------------------------
# Make binary mask for deciding mask valid lightness range
    maskimp = make_binary_mask(impZone, zoneLTh, zoneHTh, Mode, Show)
# maskimp may be 8bit or 16bit depending on original image bit
#   after deepen bit depth, Save Zone Mask
    IJ.saveAsTiff(maskimp, folder + filename +"_" + chName + "_Zone" + str(int(zoneLTh)) + "-" + str(int(zoneHTh)) + "_Mask.tif")
#--------------------------------------------------
# bind valid lightness range mask and relative color mask
    ic = plugin.ImageCalculator()
    maskBindimp = ic.run("Min created", impYmask, maskimp) # bind masks
    if Show == "Yes":
        maskBindimp.show()
#   apply Gaussian blur
    if (Gaussian == True) and (colorCh !="Luminosity"):
        IJ.run(maskBindimp, "Gaussian Blur...", "sigma=" + str(GaussianSigma))
#     save binded mask
    if MaskInvert == False: # Mask No Inversion
        IJ.saveAsTiff(maskBindimp, folder + filename +"_" + chName + "_Zone" + str(int(zoneLTh)) + "-" + str(int(zoneHTh)) + "_" + V + "_Mask_Binded.tif")
    else: # Mask Inversion
        IJ.saveAsTiff(maskBindimp, folder + filename +"_" + chName + "_Zone" + str(int(zoneLTh)) + "-" + str(int(zoneHTh)) + "_" + V + "_Mask_Binded_and_Inverted.tif")
#     close working windows
    if Show == "Yes":
        impYmask.hide()
        impZone.hide()
        maskimp.hide()
        maskBindimp.hide()
    impYmask.close()
    maskimp.close()
    maskBindimp.close()
    impZone.close()
#   End of def RGBmanipulation 
#----------------------------------------------

class ThresholdPreviewer(AdjustmentListener, ItemListener): # observe changes of dialog items
    def __init__(self, imp, slider_upper, slider_lower, checkbox_prv):
        """
             imp: an ImagePlus
             slider: a java.awt.Scrollbar UI element
             preview_checkbox: a java.awt.Checkbox controlling whether to
                                                 dynamically update the ImagePlus as the
                                                 scrollbar is updated, or not.
        """
        self.imp = imp
        self.original_ip = imp.getProcessor().duplicate() # store a copy
        self.imp_org = Duplicator().run(imp0)
        self.slider_upper = slider_upper
        self.slider_lower = slider_lower
        self.checkbox_prv = checkbox_prv

    def adjustmentValueChanged(self, event):
        """ event: an AdjustmentEvent with data on the state of the scroll bar. """
        preview = self.checkbox_prv.getState()
        if preview:
            if event.getValueIsAdjusting():
                return # there are more scrollbar adjustment events queued already
            self.threshold()

    def itemStateChanged(self, event):
        """ event: an ItemEvent with data on what happened to the checkbox. """
        if event.getStateChange() == event.SELECTED:
            self.threshold()
    
    def reset(self):
        """ Restore the original ImageProcessor """
        self.imp.setProcessor(self.original_ip)
    
    def threshold(self):
        """ Execute the in-place making binary image of the ImagePlus. """
        ip = self.imp.getProcessor()
        mask = ip.createProcessor(ip.width, ip.height)
        mask.BLACK_AND_WHITE_LUT
        mask.setValue(convValue(255, Mode))
        threshold_upper = self.slider_upper.getValue()
        threshold_lower = self.slider_lower.getValue()
        if threshold_upper < threshold_lower:
            threshold_upper = threshold_lower
        self.original_ip.setThreshold(threshold_lower, threshold_upper, ImageProcessor.OVER_UNDER_LUT)
        roi = ThresholdToSelection().convert(self.original_ip) # make ROI from threshold
        mask.setRoi(roi)
        mask.fill(roi)
        self.imp.setProcessor(mask)
        self.imp.getProcessor().setLut (None)

def thresholdImageUI(imp, Mode, C0, C1):
    ImageConverter(imp).convertToGray8()
#    imp = WM.getCurrentImage()
#    imp.plotHistogram()
    WM.getActiveWindow().toFront()
#    imppanel = ImagePanel(imp)
    gd = GenericDialog(C0 +" (" + C1 + ") Mask Maker / input threshold parameters")
    if C0 != "Luminosity Positive":
        gd.addNumericField(C0 +" (" + C1 + ") Mask Threshold \n (Signed relative value)", 10, 0)
    else:
        gd.addNumericField("This field is not available for Luminosity Mask", 0, 0)
    gd.addCheckbox("Mask Invert for " + C1 + " Mask", False)
    gd.addMessage("Decide mask valid lightness range\n         White part in preview is mask valid region")
    gd.addSlider("Upper Threshold of Mask", 0, 255, 128)
    gd.addSlider("Lower Threshold of Mask", 0, 255, 128)
    gd.addCheckbox("Preview", True)
    gd.addNumericField("Mask Transparency Factor", 1.0, 1)
    if C0 != "Luminosity Positive":
        gd.addCheckbox("Apply Gaussian Blur", False)
        gd.addNumericField("Gaussian Blur Sigma", 2.0, 1)
    showWin = ["Yes","No"]
    gd.addChoice("Show working windows?", showWin, showWin[0])
    if C0 != "Luminosity Positive":
        gd.addMessage("to make binary mask instead of grayscale mask, \n set Mask Transparency Factor = 100")
    gd.addCheckbox("Mask Inversion for Excluding selected area", False)


    # The UI elements for the above inputs
    slider_upper = gd.getSliders().get(0) # the 1st
    slider_lower = gd.getSliders().get(1) # the 2nd
    checkbox_Bmask = gd.getCheckboxes().get(0) #  chkbox for blue mask
    checkbox_prv = gd.getCheckboxes().get(1) # chkbox for preview
    nfield_maskThr = gd.getNumericFields().get(0) # 1st start
#    checkbox_Gaussian = gd.getCheckboxes().get(2) # chkbox for Gaussian Blur
#    nfield_Gaussian = gd.getNumericFields().get(1) # Gaussian Blur Sigma
    nfield_TrFac = gd.getNumericFields().get(1) # Transparency Factor
    choice_showWin = gd.getChoices().get(0)


    if not imp:
        print "Open an image first!"
        return

    previewer = ThresholdPreviewer(imp, slider_upper, slider_lower, checkbox_prv)
    slider_upper.addAdjustmentListener(previewer) # set listener for upper slider
    slider_lower.addAdjustmentListener(previewer) # set listener for lower slider
    checkbox_prv.addItemListener(previewer)  # set listener for preview checkbox
    
    gd.showDialog()

    if gd.wasCanceled():
        previewer.reset()
        Mode = "Stop"
        print "User canceled dialog!"
#    else:
#        previewer.threshold()
#    parm_Mode   = "Auto"
    YTh =  gd.getNextNumber()
    blInvert = gd.getNextBoolean()
    zoneHTh =  gd.getNextNumber()
    zoneLTh =  gd.getNextNumber()
    preview_on = gd.getNextBoolean()
    TrFac =  gd.getNextNumber()
    if C0 != "Luminosity Positive":
        Gaussian = gd.getNextBoolean()
        GaussianSigma =  gd.getNextNumber()
    else:
        Gaussian = False
        GaussianSigma =  0.0
    Show =  gd.getNextChoice()
    MaskInvert =  gd.getNextBoolean()
    imp.hide()
    imp.close()
    return Mode, YTh, blInvert, zoneLTh, zoneHTh, TrFac, str(Show), Gaussian, GaussianSigma, MaskInvert
    """
    return values
        YTh: Target Color Ch Threshold (originally Yellow Threshold)
        zoneLTh: lower threshold of mask
        zoneHTh: upper threshold of mask
        blInvert: Color invert Yes or No (originally invert for blue mask) 
        Show: working wndows Showing Mode
        TrFac: Transparency Factor
        Gaussian: Apply or not Gaussian Blur
        GaussianSigma: Gaussian Blur Sigma
        MaskInvert: Mask Inversion for excluding selected area (True / False)
    """
def selectChannel(): # dialog of selecting target channel for mask valid lightness range
    gd = GenericDialog("General Color Channel Mask Maker / Target Channel Selection")
    scolorCh = ["Luminosity", "Cyan/Red","Magenta/Green","Yellow/Blue"]
    gd.addChoice("Select target channel for making mask", scolorCh, scolorCh[3])
    sbaseCh = ["Luminosity","Red","Green","Blue"]
    gd.addChoice("Select target channel for deciding mask valid lightness range", sbaseCh, sbaseCh[3])
    gd.showDialog()

    colorCh =  gd.getNextChoice()
    baseCh =  gd.getNextChoice()
    if gd.wasCanceled():
        colorCh =  "None"
        baseCh =  "None"

    return str(baseCh),  str(colorCh)
    """
    return values
        baseCh: channel name for deciding mask upper & lower thresholds
        color Ch: channel name for making color mask image
    """
#######################################
#------------ main routine -----------#
window = WM.getActiveWindow()
Mode = "Auto"
if not window: # if no opening image > open file
    """ opening file """
    od = OpenDialog("Choose a file")
    folder = od.getDirectory()
    filename = od.getFileName()
    if filename:
        path = folder + filename
        IJ.log(path)
        IJ.log(folder)
        IJ.log(filename)
        # Always Open file with Bio-format Plugin
        imps = BF.openImagePlus(folder + filename) # Open file with Bio-format Plugin
        imp0 = imps[0] #imp0: Original Image 
        imp0.show()
    else:
        Mode = "Stop"
else: # when three is opeing file already 
    imp0 = IJ.getImage()
    filename = imp0.getTitle()
    folder = IJ.getDirectory("image") # get image's original directory
    folder = IJ.getDirectory("Output_directory")
# deciding bit depth
if Mode == "Auto":
    bitdepth = imp0.getBitDepth()
    if bitdepth == 8:
            Mode = "8-bit"
    elif bitdepth ==16:
            Mode = "16-bit"
    elif bitdepth ==32:
            Mode = "32-bit"
    else:
            Mode = "16-bit"
stackSize =imp0.getStackSize() # Color or Monochrome?
if Mode <> "Stop": # if user did not cancel execute process
    """ making preview image """
    impprv = Duplicator().run(imp0) # impprv: preview imp
    scale = 1200.0 / impprv.getWidth() # preview image scaling value (width = 1200px fixed)
    new_height = impprv.getHeight() * scale # preview image height
    new_height = int(new_height)
    tmpimpprv = impprv.resize(1200, new_height, "none")
    ImageConverter(tmpimpprv).convertToRGB() # convert to 8bit RGB
    impprvsCh = ChannelSplitter.split(tmpimpprv)
## opening parameter input dialog 1
#   parameter input for deciding base channel and color mask channel 
    if stackSize == 1: # if imp0 is Monochrome
        baseCh = "Luminosity"
        colorCh = "Luminosity"
    else: # if imp0 is RGB Color
        baseCh, colorCh = selectChannel() # select target channel for deciding upper & lower threshold
    if colorCh == "Yellow/Blue":
        C0 = "Yellow"
        C1 = "Blue"
    elif colorCh == "Cyan/Red":
        C0 = "Cyan"
        C1 = "Red"
    elif colorCh == "Magenta/Green":
        C0 = "Magenta"
        C1 = "Green"
    elif colorCh == "Luminosity":
        C0 = "Luminosity Positive"
        C1 = "Negative"
    if baseCh:
        imp0.hide()
        if baseCh == "Blue":
            impprvsCh[2].show()
            parm = thresholdImageUI(impprvsCh[2], Mode, C0, C1)
        elif baseCh == "Red":
            impprvsCh[0].show()
            parm = thresholdImageUI(impprvsCh[0], Mode, C0, C1)
        elif baseCh == "Green":
            impprvsCh[1].show()
            parm = thresholdImageUI(impprvsCh[1], Mode, C0, C1)
        else:
    #        ImageConverter(impprv).convertToRGB()
            ImageConverter(tmpimpprv).convertToGray8()
            tmpimpprv.show()
## opening parameter input dialog 2
#    threshold parameters input for making masks
#          (for valid ligheness range or color threshold)
            parm = thresholdImageUI(tmpimpprv, Mode, C0, C1)
    else: # if parameter input dialog1 was canceled
        parm = []
        parm.append("Stop")
    # closing temporary windows for preview images
    tmpimpprv.hide()
    impprvsCh[0].hide()
    impprvsCh[1].hide()
    impprvsCh[2].hide()
    tmpimpprv.close()
    impprvsCh[0].close()
    impprvsCh[1].close()
    impprvsCh[2].close()
    
    if parm[0] <> "Stop": # if parameter input dialog2 was not canceled
        """ proceeding mask making process """
        RGBmanipulation(imp0, folder, filename, baseCh, parm, colorCh, stackSize)
else: # if user canceled file opening
    IJ.log("User canceled")
IJ.log("Finish")

